@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -18,17 +18,17 @@
 
 package org.apache.zookeeper;
 
-import org.apache.yetus.audience.InterfaceAudience;
-
 import java.util.ArrayList;
 import java.util.EnumSet;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import org.apache.yetus.audience.InterfaceAudience;
 
 @SuppressWarnings("serial")
 @InterfaceAudience.Public
 public abstract class KeeperException extends Exception {
+
     /**
      * All multi-requests that result in an exception retain the results
      * here so that it is possible to examine the problems in the catch
@@ -92,65 +92,69 @@ public abstract class KeeperException extends Exception {
      */
     public static KeeperException create(Code code) {
         switch (code) {
-            case SYSTEMERROR:
-                return new SystemErrorException();
-            case RUNTIMEINCONSISTENCY:
-                return new RuntimeInconsistencyException();
-            case DATAINCONSISTENCY:
-                return new DataInconsistencyException();
-            case CONNECTIONLOSS:
-                return new ConnectionLossException();
-            case MARSHALLINGERROR:
-                return new MarshallingErrorException();
-            case UNIMPLEMENTED:
-                return new UnimplementedException();
-            case OPERATIONTIMEOUT:
-                return new OperationTimeoutException();
-            case NEWCONFIGNOQUORUM:
-               return new NewConfigNoQuorum();
-            case RECONFIGINPROGRESS:
-               return new ReconfigInProgress();
-            case BADARGUMENTS:
-                return new BadArgumentsException();
-            case APIERROR:
-                return new APIErrorException();
-            case NONODE:
-                return new NoNodeException();
-            case NOAUTH:
-                return new NoAuthException();
-            case BADVERSION:
-                return new BadVersionException();
-            case NOCHILDRENFOREPHEMERALS:
-                return new NoChildrenForEphemeralsException();
-            case NODEEXISTS:
-                return new NodeExistsException();
-            case INVALIDACL:
-                return new InvalidACLException();
-            case AUTHFAILED:
-                return new AuthFailedException();
-            case NOTEMPTY:
-                return new NotEmptyException();
-            case SESSIONEXPIRED:
-                return new SessionExpiredException();
-            case INVALIDCALLBACK:
-                return new InvalidCallbackException();
-            case SESSIONMOVED:
-                return new SessionMovedException();
-            case NOTREADONLY:
-                return new NotReadOnlyException();
-            case EPHEMERALONLOCALSESSION:
-                return new EphemeralOnLocalSessionException();
-            case NOWATCHER:
-                return new NoWatcherException();
-            case RECONFIGDISABLED:
-                return new ReconfigDisabledException();
-            case SESSIONCLOSEDREQUIRESASLAUTH:
-                return new SessionClosedRequireAuthException();
-            case REQUESTTIMEOUT:
-                return new RequestTimeoutException();
-            case OK:
-            default:
-                throw new IllegalArgumentException("Invalid exception code:" + code.code);
+        case SYSTEMERROR:
+            return new SystemErrorException();
+        case RUNTIMEINCONSISTENCY:
+            return new RuntimeInconsistencyException();
+        case DATAINCONSISTENCY:
+            return new DataInconsistencyException();
+        case CONNECTIONLOSS:
+            return new ConnectionLossException();
+        case MARSHALLINGERROR:
+            return new MarshallingErrorException();
+        case UNIMPLEMENTED:
+            return new UnimplementedException();
+        case OPERATIONTIMEOUT:
+            return new OperationTimeoutException();
+        case NEWCONFIGNOQUORUM:
+            return new NewConfigNoQuorum();
+        case RECONFIGINPROGRESS:
+            return new ReconfigInProgress();
+        case BADARGUMENTS:
+            return new BadArgumentsException();
+        case APIERROR:
+            return new APIErrorException();
+        case NONODE:
+            return new NoNodeException();
+        case NOAUTH:
+            return new NoAuthException();
+        case BADVERSION:
+            return new BadVersionException();
+        case NOCHILDRENFOREPHEMERALS:
+            return new NoChildrenForEphemeralsException();
+        case NODEEXISTS:
+            return new NodeExistsException();
+        case INVALIDACL:
+            return new InvalidACLException();
+        case AUTHFAILED:
+            return new AuthFailedException();
+        case NOTEMPTY:
+            return new NotEmptyException();
+        case SESSIONEXPIRED:
+            return new SessionExpiredException();
+        case INVALIDCALLBACK:
+            return new InvalidCallbackException();
+        case SESSIONMOVED:
+            return new SessionMovedException();
+        case NOTREADONLY:
+            return new NotReadOnlyException();
+        case EPHEMERALONLOCALSESSION:
+            return new EphemeralOnLocalSessionException();
+        case NOWATCHER:
+            return new NoWatcherException();
+        case RECONFIGDISABLED:
+            return new ReconfigDisabledException();
+        case SESSIONCLOSEDREQUIRESASLAUTH:
+            return new SessionClosedRequireAuthException();
+        case REQUESTTIMEOUT:
+            return new RequestTimeoutException();
+        case QUOTAEXCEEDED:
+            return new QuotaExceededException();
+        case THROTTLEDOP:
+            return new ThrottledOpException();
+        case OK:
+        default:
+            throw new IllegalArgumentException("Invalid exception code");
         }
     }
 
@@ -181,142 +185,143 @@ public abstract class KeeperException extends Exception {
     @Deprecated
     @InterfaceAudience.Public
     public interface CodeDeprecated {
+
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#OK} instead
          */
         @Deprecated
-        public static final int Ok = 0;
+        int Ok = 0;
 
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#SYSTEMERROR} instead
          */
         @Deprecated
-        public static final int SystemError = -1;
+        int SystemError = -1;
         /**
          * @deprecated deprecated in 3.1.0, use
          * {@link Code#RUNTIMEINCONSISTENCY} instead
          */
         @Deprecated
-        public static final int RuntimeInconsistency = -2;
+        int RuntimeInconsistency = -2;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#DATAINCONSISTENCY}
          * instead
          */
         @Deprecated
-        public static final int DataInconsistency = -3;
+        int DataInconsistency = -3;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#CONNECTIONLOSS}
          * instead
          */
         @Deprecated
-        public static final int ConnectionLoss = -4;
+        int ConnectionLoss = -4;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#MARSHALLINGERROR}
          * instead
          */
         @Deprecated
-        public static final int MarshallingError = -5;
+        int MarshallingError = -5;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#UNIMPLEMENTED}
          * instead
          */
         @Deprecated
-        public static final int Unimplemented = -6;
+        int Unimplemented = -6;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#OPERATIONTIMEOUT}
          * instead
          */
         @Deprecated
-        public static final int OperationTimeout = -7;
+        int OperationTimeout = -7;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#BADARGUMENTS}
          * instead
          */
         @Deprecated
-        public static final int BadArguments = -8;
+        int BadArguments = -8;
 
         @Deprecated
-        public static final int UnknownSession= -12;
+        int UnknownSession = -12;
 
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#NEWCONFIGNOQUORUM}
          * instead
          */
         @Deprecated
-        public static final int NewConfigNoQuorum = -13;
+        int NewConfigNoQuorum = -13;
 
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#RECONFIGINPROGRESS}
          * instead
          */
         @Deprecated
-        public static final int ReconfigInProgress= -14;
+        int ReconfigInProgress = -14;
 
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#APIERROR} instead
          */
         @Deprecated
-        public static final int APIError = -100;
+        int APIError = -100;
 
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#NONODE} instead
          */
         @Deprecated
-        public static final int NoNode = -101;
+        int NoNode = -101;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#NOAUTH} instead
          */
         @Deprecated
-        public static final int NoAuth = -102;
+        int NoAuth = -102;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#BADVERSION} instead
          */
         @Deprecated
-        public static final int BadVersion = -103;
+        int BadVersion = -103;
         /**
          * @deprecated deprecated in 3.1.0, use
          * {@link Code#NOCHILDRENFOREPHEMERALS}
          * instead
          */
         @Deprecated
-        public static final int NoChildrenForEphemerals = -108;
+        int NoChildrenForEphemerals = -108;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#NODEEXISTS} instead
          */
         @Deprecated
-        public static final int NodeExists = -110;
+        int NodeExists = -110;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#NOTEMPTY} instead
          */
         @Deprecated
-        public static final int NotEmpty = -111;
+        int NotEmpty = -111;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#SESSIONEXPIRED} instead
          */
         @Deprecated
-        public static final int SessionExpired = -112;
+        int SessionExpired = -112;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#INVALIDCALLBACK}
          * instead
          */
         @Deprecated
-        public static final int InvalidCallback = -113;
+        int InvalidCallback = -113;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#INVALIDACL} instead
          */
         @Deprecated
-        public static final int InvalidACL = -114;
+        int InvalidACL = -114;
         /**
          * @deprecated deprecated in 3.1.0, use {@link Code#AUTHFAILED} instead
          */
         @Deprecated
-        public static final int AuthFailed = -115;
-        
+        int AuthFailed = -115;
+
         // This value will be used directly in {@link CODE#SESSIONMOVED}
-        // public static final int SessionMoved = -118;       
-        
+        // public static final int SessionMoved = -118;
+
         @Deprecated
-        public static final int EphemeralOnLocalSession = -120;
+        int EphemeralOnLocalSession = -120;
 
     }
 
@@ -326,91 +331,98 @@ public abstract class KeeperException extends Exception {
      * enum values are in all CAPS.
      */
     @InterfaceAudience.Public
-    public static enum Code implements CodeDeprecated {
+    public enum Code implements CodeDeprecated {
         /** Everything is OK */
-        OK (Ok),
+        OK(Ok),
 
         /** System and server-side errors.
          * This is never thrown by the server, it shouldn't be used other than
          * to indicate a range. Specifically error codes greater than this
          * value, but lesser than {@link #APIERROR}, are system errors.
          */
-        SYSTEMERROR (SystemError),
+        SYSTEMERROR(SystemError),
 
         /** A runtime inconsistency was found */
-        RUNTIMEINCONSISTENCY (RuntimeInconsistency),
+        RUNTIMEINCONSISTENCY(RuntimeInconsistency),
         /** A data inconsistency was found */
-        DATAINCONSISTENCY (DataInconsistency),
+        DATAINCONSISTENCY(DataInconsistency),
         /** Connection to the server has been lost */
-        CONNECTIONLOSS (ConnectionLoss),
+        CONNECTIONLOSS(ConnectionLoss),
         /** Error while marshalling or unmarshalling data */
-        MARSHALLINGERROR (MarshallingError),
+        MARSHALLINGERROR(MarshallingError),
         /** Operation is unimplemented */
-        UNIMPLEMENTED (Unimplemented),
+        UNIMPLEMENTED(Unimplemented),
         /** Operation timeout */
-        OPERATIONTIMEOUT (OperationTimeout),
+        OPERATIONTIMEOUT(OperationTimeout),
         /** Invalid arguments */
-        BADARGUMENTS (BadArguments),
-        /** No quorum of new config is connected and up-to-date with the leader of last commmitted config - try 
+        BADARGUMENTS(BadArguments),
+        /** No quorum of new config is connected and up-to-date with the leader of last commmitted config - try
          *  invoking reconfiguration after new servers are connected and synced */
-        NEWCONFIGNOQUORUM (NewConfigNoQuorum),
+        NEWCONFIGNOQUORUM(NewConfigNoQuorum),
         /** Another reconfiguration is in progress -- concurrent reconfigs not supported (yet) */
-        RECONFIGINPROGRESS (ReconfigInProgress),
+        RECONFIGINPROGRESS(ReconfigInProgress),
         /** Unknown session (internal server use only) */
-        UNKNOWNSESSION (UnknownSession),
-        
+        UNKNOWNSESSION(UnknownSession),
+
         /** API errors.
          * This is never thrown by the server, it shouldn't be used other than
          * to indicate a range. Specifically error codes greater than this
          * value are API errors (while values less than this indicate a
          * {@link #SYSTEMERROR}).
          */
-        APIERROR (APIError),
+        APIERROR(APIError),
 
         /** Node does not exist */
-        NONODE (NoNode),
+        NONODE(NoNode),
         /** Not authenticated */
-        NOAUTH (NoAuth),
+        NOAUTH(NoAuth),
         /** Version conflict
-            In case of reconfiguration: reconfig requested from config version X but last seen config has a different version Y */
-        BADVERSION (BadVersion),
+         In case of reconfiguration: reconfig requested from config version X but last seen config has a different version Y */
+        BADVERSION(BadVersion),
         /** Ephemeral nodes may not have children */
-        NOCHILDRENFOREPHEMERALS (NoChildrenForEphemerals),
+        NOCHILDRENFOREPHEMERALS(NoChildrenForEphemerals),
         /** The node already exists */
-        NODEEXISTS (NodeExists),
+        NODEEXISTS(NodeExists),
         /** The node has children */
-        NOTEMPTY (NotEmpty),
+        NOTEMPTY(NotEmpty),
         /** The session has been expired by the server */
-        SESSIONEXPIRED (SessionExpired),
+        SESSIONEXPIRED(SessionExpired),
         /** Invalid callback specified */
-        INVALIDCALLBACK (InvalidCallback),
+        INVALIDCALLBACK(InvalidCallback),
         /** Invalid ACL specified */
-        INVALIDACL (InvalidACL),
+        INVALIDACL(InvalidACL),
         /** Client authentication failed */
-        AUTHFAILED (AuthFailed),
+        AUTHFAILED(AuthFailed),
         /** Session moved to another server, so operation is ignored */
-        SESSIONMOVED (-118),
+        SESSIONMOVED(-118),
         /** State-changing request is passed to read-only server */
-        NOTREADONLY (-119),
+        NOTREADONLY(-119),
         /** Attempt to create ephemeral node on a local session */
-        EPHEMERALONLOCALSESSION (EphemeralOnLocalSession),
+        EPHEMERALONLOCALSESSION(EphemeralOnLocalSession),
         /** Attempts to remove a non-existing watcher */
-        NOWATCHER (-121),
+        NOWATCHER(-121),
         /** Request not completed within max allowed time.*/
-        REQUESTTIMEOUT (-122),
+        REQUESTTIMEOUT(-122),
         /** Attempts to perform a reconfiguration operation when reconfiguration feature is disabled. */
         RECONFIGDISABLED(-123),
-        /** The session has been closed by server because server requires client to do SASL authentication,
-         *  but client is not configured with SASL authentication or configuted with SASL but failed
+        /** The session has been closed by server because server requires client to do authentication
+         *  with configured authentication scheme at the server, but client is not configured with
+         *  required  authentication scheme or configured but authentication failed
          *  (i.e. wrong credential used.). */
-        SESSIONCLOSEDREQUIRESASLAUTH(-124);
+        SESSIONCLOSEDREQUIRESASLAUTH(-124),
+        /** Exceeded the quota that was set on the path.*/
+        QUOTAEXCEEDED(-125),
+        /** Operation was throttled and not executed at all. This error code indicates that zookeeper server
+         *  is under heavy load and can't process incoming requests at full speed; please retry with back off.
+         */
+        THROTTLEDOP (-127);
 
-        private static final Map<Integer,Code> lookup
-            = new HashMap<Integer,Code>();
+        private static final Map<Integer, Code> lookup = new HashMap<Integer, Code>();
 
         static {
-            for(Code c : EnumSet.allOf(Code.class))
+            for (Code c : EnumSet.allOf(Code.class)) {
                 lookup.put(c.code, c);
+            }
         }
 
         private final int code;
@@ -422,82 +434,84 @@ public abstract class KeeperException extends Exception {
          * Get the int value for a particular Code.
          * @return error code as integer
          */
-        public int intValue() { return code; }
+        public int intValue() {
+            return code;
+        }
 
         /**
          * Get the Code value for a particular integer error code
          * @param code int error code
-         * @return Code value corresponding to specified int code, if null throws IllegalArgumentException
+         * @return Code value corresponding to specified int code, or null
          */
         public static Code get(int code) {
-            Code codeVal = lookup.get(code);
-            if (codeVal == null) {
-                throw new IllegalArgumentException("The current client version cannot lookup this code:" + code);
-            }
-            return codeVal;
+            return lookup.get(code);
         }
     }
 
     static String getCodeMessage(Code code) {
         switch (code) {
-            case OK:
-                return "ok";
-            case SYSTEMERROR:
-                return "SystemError";
-            case RUNTIMEINCONSISTENCY:
-                return "RuntimeInconsistency";
-            case DATAINCONSISTENCY:
-                return "DataInconsistency";
-            case CONNECTIONLOSS:
-                return "ConnectionLoss";
-            case MARSHALLINGERROR:
-                return "MarshallingError";
-            case NEWCONFIGNOQUORUM:
-               return "NewConfigNoQuorum";
-            case RECONFIGINPROGRESS:
-               return "ReconfigInProgress";
-            case UNIMPLEMENTED:
-                return "Unimplemented";
-            case OPERATIONTIMEOUT:
-                return "OperationTimeout";
-            case BADARGUMENTS:
-                return "BadArguments";
-            case APIERROR:
-                return "APIError";
-            case NONODE:
-                return "NoNode";
-            case NOAUTH:
-                return "NoAuth";
-            case BADVERSION:
-                return "BadVersion";
-            case NOCHILDRENFOREPHEMERALS:
-                return "NoChildrenForEphemerals";
-            case NODEEXISTS:
-                return "NodeExists";
-            case INVALIDACL:
-                return "InvalidACL";
-            case AUTHFAILED:
-                return "AuthFailed";
-            case NOTEMPTY:
-                return "Directory not empty";
-            case SESSIONEXPIRED:
-                return "Session expired";
-            case INVALIDCALLBACK:
-                return "Invalid callback";
-            case SESSIONMOVED:
-                return "Session moved";
-            case NOTREADONLY:
-                return "Not a read-only call";
-            case EPHEMERALONLOCALSESSION:
-                return "Ephemeral node on local session";
-            case NOWATCHER:
-                return "No such watcher";
-            case RECONFIGDISABLED:
-                return "Reconfig is disabled";
-            case SESSIONCLOSEDREQUIRESASLAUTH:
-                return "Session closed because client failed to authenticate";
-            default:
-                return "Unknown error " + code;
+        case OK:
+            return "ok";
+        case SYSTEMERROR:
+            return "SystemError";
+        case RUNTIMEINCONSISTENCY:
+            return "RuntimeInconsistency";
+        case DATAINCONSISTENCY:
+            return "DataInconsistency";
+        case CONNECTIONLOSS:
+            return "ConnectionLoss";
+        case MARSHALLINGERROR:
+            return "MarshallingError";
+        case NEWCONFIGNOQUORUM:
+            return "NewConfigNoQuorum";
+        case RECONFIGINPROGRESS:
+            return "ReconfigInProgress";
+        case UNIMPLEMENTED:
+            return "Unimplemented";
+        case OPERATIONTIMEOUT:
+            return "OperationTimeout";
+        case BADARGUMENTS:
+            return "BadArguments";
+        case APIERROR:
+            return "APIError";
+        case NONODE:
+            return "NoNode";
+        case NOAUTH:
+            return "NoAuth";
+        case BADVERSION:
+            return "BadVersion";
+        case NOCHILDRENFOREPHEMERALS:
+            return "NoChildrenForEphemerals";
+        case NODEEXISTS:
+            return "NodeExists";
+        case INVALIDACL:
+            return "InvalidACL";
+        case AUTHFAILED:
+            return "AuthFailed";
+        case NOTEMPTY:
+            return "Directory not empty";
+        case SESSIONEXPIRED:
+            return "Session expired";
+        case INVALIDCALLBACK:
+            return "Invalid callback";
+        case SESSIONMOVED:
+            return "Session moved";
+        case NOTREADONLY:
+            return "Not a read-only call";
+        case EPHEMERALONLOCALSESSION:
+            return "Ephemeral node on local session";
+        case NOWATCHER:
+            return "No such watcher";
+        case RECONFIGDISABLED:
+            return "Reconfig is disabled";
+        case SESSIONCLOSEDREQUIRESASLAUTH:
+            return "Session closed because client failed to authenticate";
+        case QUOTAEXCEEDED:
+            return "Quota has exceeded";
+        case THROTTLEDOP:
+            return "Op throttled due to high load";
+        default:
+            return "Unknown error " + code;
         }
     }
 
@@ -569,9 +583,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class APIErrorException extends KeeperException {
+
         public APIErrorException() {
             super(Code.APIERROR);
         }
+
     }
 
     /**
@@ -579,9 +595,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class AuthFailedException extends KeeperException {
+
         public AuthFailedException() {
             super(Code.AUTHFAILED);
         }
+
     }
 
     /**
@@ -589,12 +607,14 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class BadArgumentsException extends KeeperException {
+
         public BadArgumentsException() {
             super(Code.BADARGUMENTS);
         }
         public BadArgumentsException(String path) {
             super(Code.BADARGUMENTS, path);
         }
+
     }
 
     /**
@@ -602,12 +622,14 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class BadVersionException extends KeeperException {
+
         public BadVersionException() {
             super(Code.BADVERSION);
         }
         public BadVersionException(String path) {
             super(Code.BADVERSION, path);
         }
+
     }
 
     /**
@@ -615,9 +637,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class ConnectionLossException extends KeeperException {
+
         public ConnectionLossException() {
             super(Code.CONNECTIONLOSS);
         }
+
     }
 
     /**
@@ -625,9 +649,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class DataInconsistencyException extends KeeperException {
+
         public DataInconsistencyException() {
             super(Code.DATAINCONSISTENCY);
         }
+
     }
 
     /**
@@ -635,12 +661,14 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class InvalidACLException extends KeeperException {
+
         public InvalidACLException() {
             super(Code.INVALIDACL);
         }
         public InvalidACLException(String path) {
             super(Code.INVALIDACL, path);
         }
+
     }
 
     /**
@@ -648,9 +676,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class InvalidCallbackException extends KeeperException {
+
         public InvalidCallbackException() {
             super(Code.INVALIDCALLBACK);
         }
+
     }
 
     /**
@@ -658,9 +688,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class MarshallingErrorException extends KeeperException {
+
         public MarshallingErrorException() {
             super(Code.MARSHALLINGERROR);
         }
+
     }
 
     /**
@@ -668,9 +700,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class NoAuthException extends KeeperException {
+
         public NoAuthException() {
             super(Code.NOAUTH);
         }
+
     }
 
     /**
@@ -678,32 +712,38 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class NewConfigNoQuorum extends KeeperException {
+
         public NewConfigNoQuorum() {
             super(Code.NEWCONFIGNOQUORUM);
         }
+
     }
-    
+
     /**
      * @see Code#RECONFIGINPROGRESS
      */
     @InterfaceAudience.Public
     public static class ReconfigInProgress extends KeeperException {
+
         public ReconfigInProgress() {
             super(Code.RECONFIGINPROGRESS);
         }
+
     }
-    
+
     /**
      * @see Code#NOCHILDRENFOREPHEMERALS
      */
     @InterfaceAudience.Public
     public static class NoChildrenForEphemeralsException extends KeeperException {
+
         public NoChildrenForEphemeralsException() {
             super(Code.NOCHILDRENFOREPHEMERALS);
         }
         public NoChildrenForEphemeralsException(String path) {
             super(Code.NOCHILDRENFOREPHEMERALS, path);
         }
+
     }
 
     /**
@@ -711,12 +751,14 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class NodeExistsException extends KeeperException {
+
         public NodeExistsException() {
             super(Code.NODEEXISTS);
         }
         public NodeExistsException(String path) {
             super(Code.NODEEXISTS, path);
         }
+
     }
 
     /**
@@ -724,12 +766,14 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class NoNodeException extends KeeperException {
+
         public NoNodeException() {
             super(Code.NONODE);
         }
         public NoNodeException(String path) {
             super(Code.NONODE, path);
         }
+
     }
 
     /**
@@ -737,12 +781,14 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class NotEmptyException extends KeeperException {
+
         public NotEmptyException() {
             super(Code.NOTEMPTY);
         }
         public NotEmptyException(String path) {
             super(Code.NOTEMPTY, path);
         }
+
     }
 
     /**
@@ -750,9 +796,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class OperationTimeoutException extends KeeperException {
+
         public OperationTimeoutException() {
             super(Code.OPERATIONTIMEOUT);
         }
+
     }
 
     /**
@@ -760,9 +808,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class RuntimeInconsistencyException extends KeeperException {
+
         public RuntimeInconsistencyException() {
             super(Code.RUNTIMEINCONSISTENCY);
         }
+
     }
 
     /**
@@ -770,9 +820,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class SessionExpiredException extends KeeperException {
+
         public SessionExpiredException() {
             super(Code.SESSIONEXPIRED);
         }
+
     }
 
     /**
@@ -780,9 +832,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class UnknownSessionException extends KeeperException {
+
         public UnknownSessionException() {
             super(Code.UNKNOWNSESSION);
         }
+
     }
 
     /**
@@ -790,9 +844,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class SessionMovedException extends KeeperException {
+
         public SessionMovedException() {
             super(Code.SESSIONMOVED);
         }
+
     }
 
     /**
@@ -800,9 +856,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class NotReadOnlyException extends KeeperException {
+
         public NotReadOnlyException() {
             super(Code.NOTREADONLY);
         }
+
     }
 
     /**
@@ -810,9 +868,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class EphemeralOnLocalSessionException extends KeeperException {
+
         public EphemeralOnLocalSessionException() {
             super(Code.EPHEMERALONLOCALSESSION);
         }
+
     }
 
     /**
@@ -820,9 +880,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class SystemErrorException extends KeeperException {
+
         public SystemErrorException() {
             super(Code.SYSTEMERROR);
         }
+
     }
 
     /**
@@ -830,9 +892,11 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class UnimplementedException extends KeeperException {
+
         public UnimplementedException() {
             super(Code.UNIMPLEMENTED);
         }
+
     }
 
     /**
@@ -840,6 +904,7 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class NoWatcherException extends KeeperException {
+
         public NoWatcherException() {
             super(Code.NOWATCHER);
         }
@@ -847,6 +912,7 @@ public abstract class KeeperException extends Exception {
         public NoWatcherException(String path) {
             super(Code.NOWATCHER, path);
         }
+
     }
 
     /**
@@ -854,28 +920,60 @@ public abstract class KeeperException extends Exception {
      */
     @InterfaceAudience.Public
     public static class ReconfigDisabledException extends KeeperException {
-        public ReconfigDisabledException() { super(Code.RECONFIGDISABLED); }
+
+        public ReconfigDisabledException() {
+            super(Code.RECONFIGDISABLED);
+        }
         public ReconfigDisabledException(String path) {
             super(Code.RECONFIGDISABLED, path);
         }
+
     }
 
     /**
      * @see Code#SESSIONCLOSEDREQUIRESASLAUTH
      */
     public static class SessionClosedRequireAuthException extends KeeperException {
-        public SessionClosedRequireAuthException() { super(Code.SESSIONCLOSEDREQUIRESASLAUTH); }
+
+        public SessionClosedRequireAuthException() {
+            super(Code.SESSIONCLOSEDREQUIRESASLAUTH);
+        }
         public SessionClosedRequireAuthException(String path) {
             super(Code.SESSIONCLOSEDREQUIRESASLAUTH, path);
         }
+
     }
 
     /**
      * @see Code#REQUESTTIMEOUT
      */
     public static class RequestTimeoutException extends KeeperException {
+
         public RequestTimeoutException() {
             super(Code.REQUESTTIMEOUT);
         }
+
+    }
+
+    /**
+     * @see Code#QUOTAEXCEEDED
+     */
+    @InterfaceAudience.Public
+    public static class QuotaExceededException extends KeeperException {
+        public QuotaExceededException() {
+            super(Code.QUOTAEXCEEDED);
+        }
+        public QuotaExceededException(String path) {
+            super(Code.QUOTAEXCEEDED, path);
+        }
+    }
+
+    /**
+     * @see Code#THROTTLEDOP
+     */
+    public static class ThrottledOpException extends KeeperException {
+        public ThrottledOpException() {
+            super(Code.THROTTLEDOP);
+        }
     }
 }
