@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -18,21 +18,15 @@
 
 package org.apache.zookeeper.test;
 
-import org.junit.runners.Suite;
+import org.junit.platform.runner.JUnitPlatform;
+import org.junit.platform.suite.api.SelectClasses;
+import org.junit.runner.RunWith;
 
 /**
  * Run tests with: Netty Client against Netty server
  */
-@Suite.SuiteClasses({
-        ACLTest.class,
-        AsyncOpsTest.class,
-        ChrootClientTest.class,
-        ClientTest.class,
-        FourLetterWordsTest.class,
-        NullDataTest.class,
-        SessionTest.class,
-        WatcherTest.class,
-        ReconfigTest.class
-})
+@RunWith(JUnitPlatform.class)
+@SelectClasses({ACLTest.class, AsyncOpsTest.class, ChrootClientTest.class, ClientTest.class, FourLetterWordsTest.class, NullDataTest.class, ReconfigTest.class, SessionTest.class, WatcherTest.class})
 public class NettyNettySuiteTest extends NettyNettySuiteBase {
+
 }
